@@ -42,11 +42,23 @@ class Connector(object):
         self._response_callback = response_callback
         self._auth = None
 
+        # NOTE(TheJulia): In order to help prevent recursive post operations
+        # by allowing us to understand that we should stop authentication.
+        self._sessions_uri = None
         # NOTE(etingof): field studies reveal that some BMCs choke at
         # long-running persistent HTTP connections (or TCP connections).
         # By default, we ask HTTP server to shut down HTTP connection we've
         # just used.
         self._session.headers['Connection'] = 'close'
+        # NOTE(TheJulia): Depending on the BMC, offering compression as an
+        # acceptable response changes the ETag behavior to offering an
+        # automatic "weak" ETag response, which is appropriate because the
+        # body content *may* not be a byte for byte match given compression.
+        # Overall, the value of compression is less than the value of concise
+        # interaction with the BMC. Setting to identity basically means
+        # "without modification or compression". By default, python-requests
+        # indicates responses can be compressed.
+        self._session.headers['Accept-Encoding'] = 'identity'
 
         if username or password:
             LOG.warning('Passing username and password to Connector is '
@@ -113,7 +125,14 @@ class Connector(object):
             response = self._session.request(method, url, json=data,
                                              headers=headers,
                                              **extra_session_req_kwargs)
-        except requests.ConnectionError as e:
+        except requests.exceptions.RequestException as e:
+            # Capture any general exception by looking for the parent
+            # class of exceptions in the requests library.
+            # Specifically this will cover cases such as transport
+            # failures, connection timeouts, and encoding errors.
+            # Raising this as sushy ConnectionError allows users to
+            # understand something bad has happened, and to
+            # allow them to respond accordingly.
             raise exceptions.ConnectionError(url=url, error=e)
 
         if self._response_callback:
@@ -126,9 +145,32 @@ class Connector(object):
         try:
             exceptions.raise_for_response(method, url, response)
         except exceptions.AccessError as e:
-            if self._auth is not None and self._auth.can_refresh_session():
+            if (method == 'POST'
+                    and self._sessions_uri is not None
+                    and self._sessions_uri in url):
+                LOG.error('Authentication to the session service failed. '
+                          'Please check credentials and try again.')
+                raise
+            if self._auth is not None:
+                # self._session.auth value is only set when basic auth is used
+                if self._session.auth is not None:
+                    LOG.warning('We have encountered an AccessError when '
+                                'using \'basic\' authentication. %(err)s',
+                                {'err': str(e)})
+                    # NOTE(TheJulia): There is no way to recover Basic auth,
+                    # as we need the client to be re-launched with new
+                    # credentials.
+                    raise
                 try:
-                    self._auth.refresh_session()
+                    if self._auth.can_refresh_session():
+                        self._auth.refresh_session()
+                    else:
+                        LOG.warning('Session authentication appears to have '
+                                    'been lost at some point in time. '
+                                    'Connectivity may have been lost during '
+                                    'a prior session refresh. Attempting to '
+                                    're-authenticate.')
+                        self._auth.authenticate()
                 except exceptions.AccessError as refresh_exc:
                     LOG.error("A failure occured while attempting to refresh "
                               "the session. Error: %s", refresh_exc.message)
