@@ -514,6 +514,9 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
 
     links = LinksField('Links')
 
+    _log_resource_body = True
+    """Whether to log the whole resource body in debug mode."""
+
     def __init__(self,
                  connector,
                  path='',
@@ -575,6 +578,43 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
 
         return val
 
+    def _get_registry(self, identity, language='en', description='registry'):
+        """Get a registry with the given identity.
+
+        :param identity: The registry identity.
+        :param language: RFC 5646 language code for Message Registries.
+            Indicates language of registry to be used. Defaults to 'en'.
+        :param description: Human-readable description to use in logging.
+        :returns: the corresponding registry object or None.
+        """
+        registries = self._registries
+        if not registries:
+            LOG.info('No %s is available', description)
+            return None
+
+        for key, registry in registries.items():
+            if (registry
+                    and self._attribute_registry in (key, registry.identity)):
+                if language != registry.language:
+                    LOG.debug('Found %(descr)s but its language %(reg_lang)s '
+                              'does not match the requested %(lang)s',
+                              {'descr': description,
+                               'lang': language,
+                               'reg_lang': registry.language})
+                    continue
+
+                return registry
+
+        avail = ', '.join(f'{reg.identity} ({reg.language})'
+                          for reg in registries.values())
+        LOG.info('%(descr)s %(registry)s not available for language %(lang)s; '
+                 'available are: %(avail)s',
+                 {'descr': description,
+                  'registry': self._attribute_registry,
+                  'lang': language,
+                  'avail': avail})
+        return None
+
     def _parse_attributes(self, json_doc):
         """Parse the attributes of a resource.
 
@@ -598,11 +638,21 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
 
         :returns ETag or None
         """
-        pattern = re.compile(r'^(W\/)?("\w*")$')
-        match = pattern.match(self._get_headers().get('ETag', ''))
+        etag = self._get_headers().get('ETag')
+        if not etag:
+            return None
+
+        # Note (arne_wiebalck): in case there is a weak Etag,
+        # return it with and without the qualifier to handle
+        # vendor implementations which may require one or the
+        # other (but should actually not use weak tags in the
+        # first place).
+        pattern = re.compile(r'^(W\/)("\w*")$')
+        match = pattern.match(etag)
         if match:
-            return match.group(2)
-        return None
+            return etag + ',' + match.group(2)
+
+        return etag
 
     def _get_headers(self):
         """Returns the HTTP headers of the request for the resource.
@@ -653,7 +703,9 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         attributes = self._parse_attributes(self._json)
         LOG.debug('Received representation of %(type)s %(path)s: %(json)s',
                   {'type': self.__class__.__name__,
-                   'path': self._path, 'json': attributes})
+                   'path': self._path,
+                   'json': (attributes if self._log_resource_body
+                            else '<stripped>')})
         self._do_refresh(force)
 
         # Mark it fresh
@@ -809,3 +861,30 @@ class ResourceCollectionBase(ResourceLinksBase):
     members_identities = Field('Members', default=[],
                                adapter=utils.get_members_identities)
     """A tuple with the members identities"""
+
+
+class MutableResourceCollectionBase(ResourceCollectionBase):
+
+    def _create_member(self, values):
+        """Create a new member of this collection.
+
+        :param values: Fields to set on creation.
+        :return: Created resource or None if it was not returned by the server.
+        """
+        response = self._conn.post(self._path, data=values)
+        self.invalidate(force_refresh=True)
+
+        location = response.headers.get('Location')
+        if location is None:
+            return None
+
+        return self._resource_type(
+            self._conn, location,
+            redfish_version=self.redfish_version,
+            registries=self.registries,
+            root=self.root)
+
+    def delete_member(self, identity):
+        """Delete the given member of the collection."""
+        self._conn.delete(identity)
+        self.invalidate(force_refresh=True)
