@@ -16,8 +16,11 @@ from http import client as http_client
 import json
 from unittest import mock
 
+import requests
+
 import sushy
 from sushy import exceptions
+from sushy.resources.certificateservice import certificate
 from sushy.resources.manager import constants as mgr_cons
 from sushy.resources.manager import virtual_media
 from sushy.tests.unit import base
@@ -172,9 +175,23 @@ class VirtualMediaTestCase(base.TestCase):
             ("/redfish/v1/Managers/BMC/VirtualMedia/Floppy1"),
             data={"Image": "https://www.dmtf.org/freeImages/Sardine.img",
                   "Inserted": True, "WriteProtected": False},
-            headers={"If-Match": '"3d7b8a7360bf2941d"'})
+            headers={"If-Match": 'W/"3d7b8a7360bf2941d","3d7b8a7360bf2941d"'})
         self.assertTrue(self.sys_virtual_media._is_stale)
 
+    @mock.patch.object(requests, 'post', autospec=True)
+    def test_is_transfer_protocol_required(self, mock_post):
+        with open('sushy/tests/unit/json_samples/'
+                  'transfer_proto_required_error.json') as f:
+            response_obj = json.load(f)
+        response = mock.MagicMock()
+        response.status_code = 400
+        response.code = "Base.1.5.ActionParameterMissing"
+        response.body = response_obj['error']
+        response.raise_for_status.side_effect = requests.exceptions.HTTPError
+        mock_post.return_value = response
+        retval = self.sys_virtual_media.is_transfer_protocol_required(response)
+        self.assertTrue(retval)
+
     def test_eject_media_none(self):
         self.sys_virtual_media._actions.eject_media = None
         self.assertRaisesRegex(
@@ -222,7 +239,7 @@ class VirtualMediaTestCase(base.TestCase):
         self.sys_virtual_media._conn.patch.assert_called_once_with(
             ("/redfish/v1/Managers/BMC/VirtualMedia/Floppy1"),
             data={"Image": None, "Inserted": False},
-            headers={"If-Match": '"3d7b8a7360bf2941d"'})
+            headers={"If-Match": 'W/"3d7b8a7360bf2941d","3d7b8a7360bf2941d"'})
         self.assertTrue(self.sys_virtual_media._is_stale)
 
     def test_eject_media_pass_empty_dict_415(self):
@@ -265,3 +282,14 @@ class VirtualMediaTestCase(base.TestCase):
     def test_set_verify_certificate_wrong_input(self):
         self.assertRaises(exceptions.InvalidParameterValueError,
                           self.sys_virtual_media.set_verify_certificate, 'yes')
+
+    @mock.patch.object(certificate, 'CertificateCollection', autospec=True)
+    def test_certificate_collection(self, mock_cert_coll):
+        result = self.sys_virtual_media.certificates
+        self.assertEqual(mock_cert_coll.return_value, result)
+        mock_cert_coll.assert_called_once_with(
+            self.conn,
+            "/redfish/v1/Managers/BMC/VirtualMedia/Floppy1/Certificates/",
+            redfish_version='1.0.2',
+            registries=self.sys_virtual_media.registries,
+            root=self.sys_virtual_media.root)
